<?php

declare(strict_types=1);

namespace Rector\RectorInstaller\Tests;

use Composer\Installer\InstallationManager;
use Composer\IO\IOInterface;
use Composer\Package\PackageInterface;
use Composer\Repository\InstalledRepositoryInterface;
use PHPUnit\Framework\TestCase;
use Prophecy\Argument;
use Prophecy\PhpUnit\ProphecyTrait;
use Prophecy\Prophecy\ObjectProphecy;
use Rector\RectorInstaller\Filesystem;
use Rector\RectorInstaller\PluginInstaller;

/**
 * @covers PluginInstaller
 */
final class PluginInstallerTest extends TestCase
{
    use ProphecyTrait;

    /**
     * @var string
     */
    private const FILE_HASH = 'hash';

    /**
     * @var PluginInstaller
     */
    private $pluginInstaller;

    /**
     * @var InstalledRepositoryInterface|ObjectProphecy
     */
    private $localRepository;

    /**
     * @var IOInterface|ObjectProphecy
     */
    private $io;

    /**
     * @var InstallationManager|ObjectProphecy
     */
    private $installationManager;

    /**
     * @var ObjectProphecy|Filesystem
     */
    private $filesystem;

    /**
     * @var string
     */
    private $configurationFile;

    /**
     * @var \Composer\Util\Filesystem|ObjectProphecy
     */
    private $composerFilesystem;

    protected function setUp(): void
    {
        $this->configurationFile = __FILE__;

        $this->composerFilesystem = $this->prophesize(\Composer\Util\Filesystem::class);

        $this->filesystem = $this->prophesize(Filesystem::class);
        $this->filesystem->isFile($this->configurationFile)
            ->shouldBeCalledOnce()
            ->willReturn(true);
        $this->filesystem->hashFile($this->configurationFile)
            ->willReturn(self::FILE_HASH);

        $this->localRepository = $this->prophesize(InstalledRepositoryInterface::class);

        $this->io = $this->prophesize(IOInterface::class);

        $this->installationManager = $this->prophesize(InstallationManager::class);

        $this->pluginInstaller = new PluginInstaller(
            $this->filesystem->reveal(),
            $this->localRepository->reveal(),
            $this->io->reveal(),
            $this->installationManager->reveal(),
            $this->composerFilesystem->reveal(),
            $this->configurationFile
        );
    }

    public function testNoRectorPackagesInstalled(): void
    {
        $this->localRepository->getPackages()
            ->willReturn([]);
        $this->filesystem->writeFile($this->configurationFile, Argument::any())->shouldNotBeCalled();
        $this->filesystem->hashEquals(self::FILE_HASH, Argument::any())->willReturn(true);
        $this->pluginInstaller->install();
    }

    public function testPackagesInstalled(): void
    {
        $rectorExtensionPackage = $this->prophesize(PackageInterface::class);
        $rectorExtensionPackage->getType()
            ->willReturn(PluginInstaller::RECTOR_EXTENSION_TYPE);
        $rectorExtensionPackage->getName()
            ->willReturn('rector/doctrine');
        $rectorExtensionPackage->getFullPrettyVersion()
            ->willReturn('rector/doctrine');
        $rectorExtensionPackage->getExtra()
            ->willReturn([
                PluginInstaller::RECTOR_EXTRA_KEY => [
                    'includes' => ['config/config.php'],
                ],
            ]);

        $nonRectorExtensionPackage = $this->prophesize(PackageInterface::class);

        $nonRectorExtensionPackageWithExtra = $this->prophesize(PackageInterface::class);
        $nonRectorExtensionPackageWithExtra->getType()
            ->willReturn(null);
        $nonRectorExtensionPackageWithExtra->getName()
            ->willReturn('rector/foo');
        $nonRectorExtensionPackageWithExtra->getFullPrettyVersion()
            ->willReturn('rector/foo');
        $nonRectorExtensionPackageWithExtra->getExtra()
            ->willReturn([
                PluginInstaller::RECTOR_EXTRA_KEY => [
                    'includes' => ['config/config.php'],
                ],
            ]);

        $packages = [$rectorExtensionPackage, $nonRectorExtensionPackage, $nonRectorExtensionPackageWithExtra];

        $this->io->write('<info>ssch/rector-extension-installer:</info> Extensions installed')
            ->shouldBeCalledOnce();
        $this->io->write(sprintf('> <info>%s:</info> installed', 'rector/doctrine'))
            ->shouldBeCalledOnce();
        $this->io->write(sprintf('> <info>%s:</info> installed', 'rector/foo'))
            ->shouldBeCalledOnce();

        $this->installationManager->getInstallPath($rectorExtensionPackage)
            ->shouldBeCalledOnce();
        $this->installationManager->getInstallPath($nonRectorExtensionPackageWithExtra)
            ->shouldBeCalledOnce();

        $this->filesystem->hashEquals(self::FILE_HASH, Argument::any())->willReturn(false);
        $this->filesystem->writeFile($this->configurationFile, Argument::any())->shouldBeCalledOnce();

        $this->localRepository->getPackages()
            ->willReturn($packages);
        $this->pluginInstaller->install();
    }
}
